@@ -12,6 +12,8 @@ import tarfile
 import tempfile
 import time
 import traceback
+import urllib
+
 from timeit import default_timer as timer
 from typing import Any, AnyStr, cast, Dict, IO, Iterable, List, Optional, Tuple, Union
 from dataclasses import dataclass
@@ -510,8 +512,11 @@ class SQLiteIndexedTar(MountSource):
         if not self.indexFilePath or self.indexFilePath == ':memory:' or not self.sqlConnection:
             return
 
+        self.sqlConnection.commit()
         self.sqlConnection.close()
-        self.sqlConnection = SQLiteIndexedTar._openSqlDb(f"file:{self.indexFilePath}?mode=ro", uri=True)
+
+        uriPath = urllib.parse.quote(self.indexFilePath)
+        self.sqlConnection = SQLiteIndexedTar._openSqlDb(f"file:{uriPath}?mode=ro", uri=True)
 
     @staticmethod
     def _tarInfoFullMode(tarInfo: tarfile.TarInfo) -> int:
@@ -737,6 +742,8 @@ class SQLiteIndexedTar(MountSource):
                 self._updateProgressBar(progressBar, fileObject)
             fileSize = fileObject.tell()
 
+            mode = 0o777 | stat.S_IFREG  # default mode
+
             # fmt: off
             fileInfo = (
                 ""                                   ,  # 0 path
@@ -745,7 +752,7 @@ class SQLiteIndexedTar(MountSource):
                 0                                    ,  # 3 data offset
                 fileSize                             ,  # 4
                 tarInfo.st_mtime if tarInfo else 0   ,  # 5
-                tarInfo.st_mode if tarInfo else 0o777,  # 6
+                tarInfo.st_mode if tarInfo else mode ,  # 6
                 None                                 ,  # 7 TAR file type. Currently unused. Overlaps with mode
                 None                                 ,  # 8 linkname
                 tarInfo.st_uid if tarInfo else 0     ,  # 9
@@ -1299,8 +1306,17 @@ class SQLiteIndexedTar(MountSource):
             )
 
         if compression == 'gz':
+            # The buffer size must be much larger than the spacing or else there will be large performance penalties
+            # even for reading sequentially, see https://github.com/pauldmccarthy/indexed_gzip/issues/89
+            # Use 4x spacing because each raw read seeks from the last index point even if the position did not change
+            # since the last read call. On average, this incurs an overhead of spacing / 2. For 3x spacing, this
+            # overhead would be 1/6 = 17%, which should be negligible. The increased memory-usage is not an issue
+            # because internally many buffers are allocated with 4 * spacing size.
+            bufferSize = max(3 * 1024 * 1024, 3 * gzipSeekPointSpacing)
             # drop_handles keeps a file handle opening as is required to call tell() during decoding
-            tar_file = indexed_gzip.IndexedGzipFile(fileobj=fileobj, drop_handles=False, spacing=gzipSeekPointSpacing)
+            tar_file = indexed_gzip.IndexedGzipFile(
+                fileobj=fileobj, drop_handles=False, spacing=gzipSeekPointSpacing, buffer_size=bufferSize
+            )
         elif compression == 'bz2':
             tar_file = indexed_bzip2.open(fileobj, parallelization=parallelization)
         else:
@@ -1349,9 +1365,12 @@ class SQLiteIndexedTar(MountSource):
             try:
                 offsets = dict(db.execute(f"SELECT blockoffset,dataoffset FROM {table_name};"))
                 fileObject.set_block_offsets(offsets)
-            except Exception:
+            except Exception as exception:
                 if self.printDebug >= 2:
                     print(f"[Info] Could not load {self.compression} block offset data. Will create it from scratch.")
+                    print(exception)
+                if self.printDebug >= 3:
+                    traceback.print_exc()
 
                 tables = [x[0] for x in db.execute('SELECT name FROM sqlite_master WHERE type="table";')]
                 if table_name in tables:
